'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
const TYPE_STRING = exports.TYPE_STRING = 'S';
const TYPE_NUMBER = exports.TYPE_NUMBER = 'N';
const TYPE_BOOLEAN = exports.TYPE_BOOLEAN = 'BOOL';
const TYPE_LIST = exports.TYPE_LIST = 'L';
const TYPE_MAP = exports.TYPE_MAP = 'M';
const TYPE_BINARY = exports.TYPE_BINARY = 'B';
const TYPE_STRING_SET = exports.TYPE_STRING_SET = 'SS';
const TYPE_NUMBER_SET = exports.TYPE_NUMBER_SET = 'NS';
const TYPE_BINARY_SET = exports.TYPE_BINARY_SET = 'BS';
const TYPE_NULL = exports.TYPE_NULL = 'NULL';

// Dynamodb key type for partition key
const HASH = exports.HASH = 'HASH';

// Dynamodb key type for sort key
const RANGE = exports.RANGE = 'RANGE';

class Field {
  constructor(name) {
    this.name = name; // The name of the field
    this.isRequired = false; // Flag to ensure if the attribute is required
    this.attributeType = TYPE_STRING; // The type of the attribute, string by default

    this.globalIndex = null; // An object to define global secondary index name and type
    this.localIndex = null; // Local secondary index name on this field

    this.hasttl = false; // Time to live marker attrbute
  }

  number() {
    this.attributeType = TYPE_NUMBER;
    return this;
  }

  boolean() {
    this.attributeType = TYPE_BOOLEAN;
    return this;
  }

  list() {
    this.attributeType = TYPE_LIST;
    return this;
  }

  map() {
    this.attributeType = TYPE_MAP;
    return this;
  }

  binary() {
    this.attributeType = TYPE_BINARY;
    return this;
  }

  set() {
    if (this.attributeType === TYPE_STRING) {
      this.attributeType = TYPE_STRING_SET;
    } else if (this.attributeType === TYPE_NUMBER) {
      this.attributeType = TYPE_NUMBER_SET;
    } else if (this.attributeType === TYPE_BINARY) {
      this.attributeType = TYPE_BINARY_SET;
    } else {
      throw new Error(`Invalid scalar type to declare set for ${this.name}`);
    }
  }

  required() {
    this.isRequired = true;
    return this;
  }

  global(name, sort = false, readCapacityUnits = 5, writeCapacityUnits = 1) {
    this.globalIndex = {
      name,
      type: sort ? RANGE : HASH,
      readCapacityUnits,
      writeCapacityUnits
    };
    return this;
  }

  local(name) {
    this.localIndex = name;
    return this;
  }

  ttl() {
    this.hasttl = true;
    this.attributeType = TYPE_NUMBER;
    return this;
  }
}

exports.default = Field;