'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.doc = exports.db = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createCollection;

var _dynamodb = require('aws-sdk/clients/dynamodb');

var _dynamodb2 = _interopRequireDefault(_dynamodb);

var _createTable = require('./createTable');

var _createTable2 = _interopRequireDefault(_createTable);

var _deleteTable = require('./deleteTable');

var _deleteTable2 = _interopRequireDefault(_deleteTable);

var _updateTTL = require('./updateTTL');

var _updateTTL2 = _interopRequireDefault(_updateTTL);

var _insert = require('./insert');

var _insert2 = _interopRequireDefault(_insert);

var _findOne = require('./findOne');

var _findOne2 = _interopRequireDefault(_findOne);

var _update = require('./update');

var _update2 = _interopRequireDefault(_update);

var _delete = require('./delete');

var _delete2 = _interopRequireDefault(_delete);

var _Key = require('./_Key');

var _Key2 = _interopRequireDefault(_Key);

var _Field = require('./_Field');

var _Field2 = _interopRequireDefault(_Field);

var _validateSchema = require('./_validateSchema');

var _validateSchema2 = _interopRequireDefault(_validateSchema);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// AWS dynamodb options, set via environment variables
const options = {
  region: process.env.AWS_REGION,
  endpoint: process.env.DYNAMODB_ENDPOINT || undefined
};

const NAME_PREFIX = process.env.DYNAMODB_PREFIX || '';

// The dynamodb instance
const db = exports.db = new _dynamodb2.default(options);
const doc = exports.doc = new _dynamodb2.default.DocumentClient(options);

// Schema generator, for declaring schema
const schemaGenerator = {
  key: name => new _Key2.default(name),
  field: name => new _Field2.default(name)
};

// Default schema, with auto generated id, similar to mongodb
const DEFAULT_SCHEMA = [new _Key2.default('_id').auto()];

function createCollection(name, schemaFn, procs) {
  const prefixedName = `${NAME_PREFIX}${name}`;

  const schema = schemaFn ? schemaFn(schemaGenerator) : DEFAULT_SCHEMA;

  const schemaDef = (0, _validateSchema2.default)(schema, prefixedName);

  // The generic collection object, with standard functions
  const collection = {
    name: prefixedName,
    createTable: (0, _createTable2.default)(db, prefixedName, schemaDef),
    deleteTable: (0, _deleteTable2.default)(db, prefixedName, schemaDef),
    updateTTL: (0, _updateTTL2.default)(db, prefixedName, schemaDef),
    insert: (0, _insert2.default)(doc, prefixedName, schemaDef),
    upsert: (0, _insert2.default)(doc, prefixedName, schemaDef, true),
    update: (0, _update2.default)(doc, prefixedName, schemaDef),
    delete: (0, _delete2.default)(doc, prefixedName, schemaDef),
    findOne: (0, _findOne2.default)(doc, prefixedName, schemaDef)
  };

  if (!procs) {
    return collection;
  }

  return _extends({}, collection, procs({ db, doc, self: collection }));
}